//
// Created by farresti on 21/08/18.
//


#include <SDL_image.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h> 
#include "display.h"
#include "common.h"
#include "squeezenet.h"
#include "labels.h"
#include "utility.h"
#include "md5.h"


int nbDump = 0;

uint64_t dumps[MAX_DUMP]; 

double getElapsedNanoSec(uint64_t *start, uint64_t *end) {
  return ((*end) - (*start));
}

int dumpTime(uint64_t * dumpBuffer) {
  struct timespec clock;
  clock_gettime(CLOCK_MONOTONIC, &clock);
  dumpBuffer[nbDump++] = (uint64_t) (clock.tv_sec * 1e9) + clock.tv_nsec;
  return nbDump - 1;
}


void dumpAndPrintTime(int previous, uint64_t * dumpBuffer, const char* funcName) {
  int current = dumpTime(dumpBuffer);
  double t = getElapsedNanoSec(dumps+previous, dumps+current);
  fprintf(stderr, "Function <%s>: %.0lf (ns)\n", funcName, t);
}




static void displayResult(int image_width, int image_height, int best) {
  // 4.1 Initialize SDL
  display_t disp;
  initDisplay(image_width, image_height, &disp);

  // 4.2 Display
  disp.image = IMG_Load(DEFAULT_PATH_IMG);
  displayWithText(&disp, squeezenet_labels[best]);
  printf("Press Any Key to Continue\n");
  getchar();
  endDisplay(&disp);
}

int main(int argc, char *argv[]) {
  for (int i = 0; i < 3; i++) {

    // 1. Get and load user image
    int image_depth = 0;
    int image_width = 0;
    int image_height = 0;



    int d = dumpTime(dumps);
    float *image = loadImageF32(DEFAULT_PATH_IMG,
                                &image_depth, &image_width, &image_height);
    dumpAndPrintTime(d, dumps, "loadImage");
    
    if (!image) {
      return -1;
    }

    // 2. Load network parameters
    float *weights = NULL;
    float *biases = NULL;
    d = dumpTime(dumps);
    loadRawWeightsAndBiases(DEFAULT_PATH_WEIGHTS,
                            DEFAULT_PATH_BIASES,
                            &weights, &biases);
    dumpAndPrintTime(d, dumps, "loadWeights");

    // 3. Do inference
    struct timespec start, finish;
    double elapsed;
     
    clock_gettime(CLOCK_MONOTONIC, &start);
    double result[N_LABELS] = {0};
    if (squeezenetPredict(image, image_width, image_height, image_depth, weights, biases, result) < 0) {
      return -1;
    }
    int best_result = squeezenetDecode(N_LABELS, result);

    d = dumpTime(dumps);
    MD5_Update(N_LABELS * sizeof(double), result);
    dumpAndPrintTime(d, dumps, "MD5");


    clock_gettime(CLOCK_MONOTONIC, &finish);

    elapsed = (finish.tv_sec - start.tv_sec);
    elapsed += (finish.tv_nsec - start.tv_nsec) / 1000000000.0;
    fprintf(stderr, "Total time: %lf seconds\n",elapsed);

    // 4.  Display result
    //    displayResult(image_width, image_height, best_result);

    // 5. Clean everything
    free(weights);
    free(biases);
    free(image);

    nbDump = 0;
  }
  return 0;
}
