/*
	============================================================================
	Name        : BP1D.c
	Author      : amercat
	Author      : ggautier
	Version     : 1.0
	Copyright   : CECILL-C
	Description : code for BP1D selection.
	============================================================================
*/
#include <stdlib.h>
#include "costMatrix.h"
#include "BP1D.h"
#include "bp1d_node.h"


node_type min2(node_type val1, node_type val2)
{
    return val1<val2?(val1):(val2);
}

node_type min3(node_type val1, node_type val2, node_type val3)
{
    return min2(min2(val1,val2),val3);
}

// #codeOriginalDegueulasse

// BP_ComputeFi computes the forward pass
// forward messages are stored in the Fi table of node structure
void BP_ComputeFi(IN phib_type *phiB, OUT fi_type *Fi, int width, int ndisp, int Phi0, int nbSlice, int height)
{

	int x, dd, y;
	int maxY = height/nbSlice;
	fi_type * outFi = Fi;
	phib_type * inPhiB = phiB;

	size_t offsetFi = (width-(ndisp-1)/2-1)*(ndisp+2);
	size_t offsetPhiB = (width-(ndisp-1)/2-1)*(ndisp+1);

	for (y = 0; y < maxY; y++) {

		node_type MR_MR, B_MR;
		node_type ML_B, B_B, MR_B;

		int halfdispartity = (ndisp-1)/2;
		int width_inPhiB = (width-(halfdispartity + 1));

		for (dd=0; dd < ndisp + 2; dd++)						// initialisation of the outFi outFirst colomn to 0
		{
			outFi[dd].M = 0;
			outFi[dd].B  = 0;
		}

		for (x=1; x < width_inPhiB; x++)						// initialisation of the outFi outFirst lign and last lign
		{
			size_t offset = x*(ndisp + 2);
			outFi[offset + ndisp + 1].M = F_BIG;		// normalement inutile	=> pas d'après les tests !
			outFi[offset + ndisp + 1].B = F_BIG;		// normalement inutile	=> pas d'après les tests !
			outFi[offset].M				= F_BIG;		// normalement inutile	=> pas d'après les tests !
			outFi[offset].B				= F_BIG;		// peut être inutile ?
		}


		for (x=1; x < width_inPhiB; x++)

		{

			size_t offsetXmin1 = (x-1)*(ndisp + 2);
			size_t offsetXmin11 = (x-1)*(ndisp + 1);
			size_t offsetX = x*(ndisp+2);

			// compute outFi for the disparity d = 0
			// outFi->ML
			MR_MR	= outFi[offsetXmin1 + 2].M	+ Phi0;
			B_MR	= outFi[offsetXmin1 + 2].B	+ inPhiB[offsetXmin11 + 1];
			outFi[offsetX+1].M  = min2 (MR_MR , B_MR);

			// outFi->B
			MR_B	= outFi[offsetXmin1+1].M		+ Phi0 ;
			B_B		= outFi[offsetXmin1+1].B		+ inPhiB[offsetXmin11];
			ML_B	= outFi[offsetXmin1].B			+ Phi0;
			outFi[offsetX+1].B  = min3 (ML_B , B_B , MR_B);

			for (dd=0; dd < halfdispartity; dd++)
			{

				size_t dd2 = dd*2;
				size_t offsetXmin1dd2 = offsetXmin1 + dd2;
				size_t offsetXmin11dd2 = offsetXmin11 + dd2;
				size_t offsetXdd2 = offsetX + dd2;

				// compute outFi even disparities (d = dd*2) using previous outFi and Phi_B
				// values for d = 0 and d = 2 are required to compute d = 1
				// outFi->ML
				MR_MR	= outFi[offsetXmin1dd2 + 4].M	+ Phi0;
				B_MR	= outFi[offsetXmin1dd2 + 4].B	+ inPhiB[offsetXmin11dd2 + 3];
				outFi[offsetXdd2 + 3].M  = min2 (MR_MR , B_MR);

				// outFi->B
				MR_B	= outFi[offsetXmin1dd2 + 3].M 	+ Phi0;
				B_B		= outFi[offsetXmin1dd2 + 3].B	+ inPhiB[offsetXmin11dd2 + 2];
				ML_B	= outFi[offsetXmin1dd2 + 2].B	+ Phi0;
				outFi[offsetXdd2 + 3].B  = min3( ML_B , B_B , MR_B);


				// compute outFi for odd disparities (d = 2*dd + 1) using previous outFi and Phi_B
				// outFi->ML
				MR_MR	= outFi[offsetXdd2 + 3].M	+ Phi0;
				B_MR = outFi[offsetXdd2 + 3].B		+ inPhiB[x*(ndisp + 1) + dd2 + 2];
				outFi[offsetXdd2 + 2].M = min2(MR_MR, B_MR);

				// outFi->B
				MR_B = outFi[offsetXmin1dd2 + 2].M	+ Phi0;
				B_B = outFi[offsetXmin1dd2 + 2].B	+ inPhiB[offsetXmin11dd2 + 1];
				ML_B = outFi[offsetXdd2 + 1].B		+ Phi0;
				outFi[offsetXdd2 + 2].B = min3(ML_B, B_B, MR_B);
			}
		}

		outFi += offsetFi;
		inPhiB += offsetPhiB;
    }
}

// #codeOriginalDegueulasse

// BP_Compute_Energy computes the backward pass = energy for each node of the BVP
// backward messages are stored in the GrandPhi table of node structure
// the energies are stored in the table Energy but it is not required
void BP_Compute_Energy(IN phib_type *phiB, IN fi_type *Fi, unsigned char *disparity_map, int height, int width, int ndisp, int Phi0, int nbSlice)
{
	int x, dd, y;
	struct node* GrandPhi;
	node_type ML_ML, B_ML;
	node_type MR_B, B_B, ML_B;
	energy_type Energy_B, Energy_ML, Energy_MR, min;

	int halfdisparity = (ndisp-1)/2;
	int width_inPhiB = (width-(halfdisparity + 1));
	energy_type* Energy = malloc(sizeof(energy_type)*( width_inPhiB * ndisp ));
	GrandPhi = malloc(sizeof(struct node)*(ndisp+2));

	fi_type * inFi = Fi;
	phib_type * inPhiB = phiB;
	unsigned char * outDisp = disparity_map;

	size_t offsetFi = (width-(ndisp-1)/2-1)*(ndisp+2);
	size_t offsetPhiB = (width-(ndisp-1)/2-1)*(ndisp+1);
	size_t offsetDisp = width;

	int maxY = height/nbSlice;
	for (y = 0; y < maxY; y++) {

		for (dd=0; dd < ndisp + 1; dd++)						// initialisation of the GrandPhi backward message inFirst colomn to 0
		{
			GrandPhi[dd+1].M	= 0;
			GrandPhi[dd+1].B	= 0;
		}

		GrandPhi[0].M			= F_BIG;
		GrandPhi[0].B			= F_BIG;

		GrandPhi[ndisp+1].M	= F_BIG;
		GrandPhi[ndisp+1].B	= F_BIG;


		size_t winPhiBmin1d1 = (width_inPhiB - 1)*(ndisp + 1);
		size_t winPhiBmin1d2 = (width_inPhiB - 1)*(ndisp + 2);
		size_t winPhiBmin1d = (width_inPhiB - 1)*ndisp;

		for (dd=0; dd < ndisp; dd++)						// compute energies for the last column "width_inPhiB"
		{
			Energy_B	= inPhiB[winPhiBmin1d1 + dd]	+ inFi[winPhiBmin1d2 + dd].B;
			Energy_ML	= Phi0							+ inFi[winPhiBmin1d2 + dd].B;
			Energy_MR	= Phi0							+ inFi[winPhiBmin1d2 + dd].M;
			Energy[winPhiBmin1d + dd]  = min3 (Energy_B , Energy_ML , Energy_MR);
		}

		for (x=width_inPhiB-2; x>=0; x--)
		{

			size_t xPls1d1 = (x+1)*(ndisp+1);
			size_t xd2 = x*(ndisp+2);
			size_t xd1 = x*(ndisp+1);
			size_t xd = x*ndisp;
			for (dd= 0; dd < halfdisparity ; dd++)
			{
				size_t dd2 = dd*2;
				// compute GrandPhi message for odd disparities (d = 2*dd + 1) using previous GrandPhi and Phi_B
				// should start with odd disparities before even ones
				// GrandPhi->MR
				ML_ML	=	GrandPhi[dd2 + 3].M		+	Phi0;
				B_ML	=	GrandPhi[dd2 + 3].B		+	inPhiB[xPls1d1 + dd2 + 2];


				// GrandPhi->B
				MR_B	=	GrandPhi[dd2 + 1 ].B 		+	Phi0;
				B_B		=	GrandPhi[dd2 + 2 ].B		+	inPhiB[xPls1d1 + dd2 + 1];
				ML_B	=	GrandPhi[dd2 + 2 ].M 		+	Phi0;

				GrandPhi[dd2 + 2].M = min2 (ML_ML, B_ML); // Update : GrandPhi after computing ML_B
				GrandPhi[dd2 + 2].B = min3 (ML_B , B_B , MR_B);


				Energy_B	= GrandPhi[dd2 + 2].B	+ inPhiB[xd1 + dd2 + 1]	+ inFi[xd2 + dd2 + 2].B;
				Energy_ML	= GrandPhi[dd2 + 2].M		+ Phi0				+ inFi[xd2 + dd2 + 2].B;
				Energy_MR	= GrandPhi[dd2 + 2].B		+ Phi0				+ inFi[xd2 + dd2 + 2].M;
				Energy[xd + dd2 + 1]  = min3 (Energy_B , Energy_ML , Energy_MR);

				// compute GrandPhi message for even disparities (d = 2*dd) using previous GrandPhi and Phi_B
				// GrandPhi->MR
				ML_ML	=	GrandPhi[dd2 + 2].M		+	Phi0;
				B_ML	=	GrandPhi[dd2 + 2].B		+	inPhiB[xd1 + dd2 + 1];


				// GrandPhi->B
				MR_B	= GrandPhi[dd2].B 			+	Phi0;
				B_B		= GrandPhi[dd2 + 1].B		+	inPhiB[xPls1d1 + dd2];
				ML_B	= GrandPhi[dd2 + 1].M 		+	Phi0;

				GrandPhi[dd2 + 1].M = min2 (ML_ML, B_ML); // Update : GrandPhi after computing ML_B
				GrandPhi[dd2 + 1].B = min3 (ML_B , B_B , MR_B);

				Energy_B =	GrandPhi[dd2 + 1].B	+ inPhiB[xd1 + dd2]		+ inFi[xd2 + dd2 + 1].B;
				Energy_ML = GrandPhi[dd2 + 1].M	+ Phi0					+ inFi[xd2 + dd2 + 1].B;
				Energy_MR = GrandPhi[dd2 + 1].B	+ Phi0					+ inFi[xd2 + dd2 + 1].M;

				Energy[xd + dd2] = min3 (Energy_B , Energy_ML , Energy_MR);

			}

			// compute GrandPhi for the last disparity d = 2 * halfdisparity+1
			// 2 * dd + 1 = ndisp
			// GrandPhi->MR

			ML_ML	=	GrandPhi[ndisp + 1].M		+ Phi0;
			B_ML	=	GrandPhi[ndisp + 1].B		+ inPhiB[xd1 + ndisp];


			// GrandPhi->B
			MR_B	=	GrandPhi[ndisp - 1].B		+	Phi0;
			B_B		=	GrandPhi[ndisp].B			+	inPhiB[xPls1d1 + ndisp];
			ML_B	=	GrandPhi[ndisp].M 		+	Phi0;
			GrandPhi[ndisp].M = min2 (ML_ML, B_ML);  // Update : GrandPhi after computing ML_B
			GrandPhi[ndisp].B  = min3 (ML_B , B_B , MR_B);

			Energy_B	=	GrandPhi[ndisp].B		+ inPhiB[xd1 + ndisp - 1]	+ inFi[xd2 + ndisp].B;
			Energy_ML	=	GrandPhi[ndisp].M		+ Phi0						+ inFi[xd2 + ndisp].B;
			Energy_MR	=	GrandPhi[ndisp].B		+ Phi0						+ inFi[xd2 + ndisp].M;
			Energy[xd + ndisp - 1] = min3 (Energy_B , Energy_ML , Energy_MR);


		}

		for (x = width_inPhiB-1; x >= 0; x--)
		{
			size_t xd = x*ndisp;
			min = Energy[xd];
			outDisp[x] = 0;
			for (dd = 1; dd < ndisp; dd++)
			{
				if (Energy[xd + dd] < min)
				{
					min = Energy[xd + dd];
					outDisp[x] = dd  ;
					outDisp[x] = outDisp[x] ;
				}
			}
		}

		inPhiB += offsetPhiB;
		inFi += offsetFi;
		outDisp += offsetDisp;

	}

	free(Energy);
	free(GrandPhi);
}
