/*
	============================================================================
	Name        : costMatrix.c
	Author      : ggautier
	Version     : 1.0
	Copyright   : CECILL-C
	Description : code to slice a cost matrix.
	============================================================================
*/
#include "costMatrix.h"
#include "image_load.h"
#include <string.h>
#include <stdio.h>


void costMatrix_dxy_getDisparity(int height,
								 int width,
								 int ndisp,
								 IN uint8_t *d,
								 IN const uint8_t* CostIN,
								 OUT uint8_t* CostOUT)
{
	int x, y;
	int pdisp, pmat;

	for(y=0, pdisp = 0; y<height; y++)
	{
		pmat = y * width * ndisp + *d;
		for(x=0; x<width; x++, pdisp++)
		{
			CostOUT[pdisp] = CostIN[pmat];
			pmat += ndisp;
		}
	}
}

void counterSlice(int nbSlice, OUT int * counterSlice) {
	int idx;
	for (idx = 0; idx < nbSlice; idx++) {
		counterSlice[idx] = idx;
	}
}


/*Only use this function for cyclopean view*/
void costMatrix_dxy_getLine(int height,
			    int width,
			    int ndisp,
				int nbSlice,
				IN int * counterSlice,
			    IN const uint8_t* CostIN,
			    OUT uint8_t* phiB)
{
	int x, y, dd;
	int sizeSlice = height/nbSlice;
	int minY = (*counterSlice)*sizeSlice;
	int halfdispartity = (ndisp-1)/2;
	int width_phib = (width-(halfdispartity + 1));

	size_t size = height*width;
	size_t offsetInSlice = minY*width;
	size_t widthPhiD = width_phib*(ndisp+1);
    for (y = 0; y < sizeSlice; y++) {

    	size_t yWidthPhiD = y*widthPhiD;
    	size_t yWidth = y*width + offsetInSlice;
    	for (x = 0; x < width_phib; x++) {

    		size_t xd1 = x * (ndisp+1);
            for (dd = 0; dd < halfdispartity + 1; dd++) {

            	size_t dd2 = dd*2;
            	size_t dd2size = dd2*size;
            	size_t xdd2dPls1 = xd1 + dd2;
            	size_t offsetIn = yWidth + halfdispartity + x + dd2 + dd2size;
                phiB[yWidthPhiD + xdd2dPls1] = 		CostIN[offsetIn];
                phiB[yWidthPhiD + xdd2dPls1 + 1] = 	CostIN[offsetIn + 1 + size];
            }
        }
    }
}


void costMatrix_debug_dxy_u8(int height, int width, int ndisp, IN const uint8_t* costs, int prefix, OUT uint8_t* costsOUT){
	uint8_t min, max, cur;
	size_t d, p;
	char* str;
	uint8_t* buff;

	min = UINT8_MAX;
	max = 0;

	str = malloc(sizeof(char)*200);

    struct str_image_gray curd;
    image_gray_init(&curd,width,height);

    for(p=0; p<(height*width*ndisp); p++)
	{
		cur = costs[p];
		min = (cur < min) ? cur : min;
		max = (cur > max) ? cur : max;
		costsOUT[p]=costs[p];
	}

	buff = malloc(sizeof(buff[0]) * width * height);

	for(d=0; d<ndisp; d++)
	{
		sprintf(str, "./../../dat/%d_debug_%d.ppm", prefix, (int)d);
        memcpy(curd.g,costs+width*height*d,sizeof(uint8_t)*width*height);
        image_gray_save(str,&curd);
	}
    image_gray_destroy(&curd);

	free(str);
	free(buff);
}


