/*
	============================================================================
	Name        : rgb2Gray.c
	Author      : kdesnos
	Version     : 1.0
	Copyright   : CECILL-C, IETR, INSA Rennes
	Description : Transformation of an RGB image into a gray-level image.
	============================================================================
*/

#include "rgb2Gray.h"

#define RGB2GRAY_COEF_R 0.29893602129378
#define RGB2GRAY_COEF_G 0.58704307445112
#define RGB2GRAY_COEF_B 0.11402090425510

void rgb2Gray(int size, int nbSlice, unsigned char *rgb, uint8_t *gray) {
    size_t idx;
    size_t offset = 0;
    size_t idxMax = size/nbSlice;
    if (gray != NULL) {
    	for (idx = 0; idx < idxMax; idx++) {
    		rgb[idx] = RGB2GRAY_COEF_R*(float)rgb[offset] +
    					RGB2GRAY_COEF_G*(float)rgb[offset+1] +
                    	RGB2GRAY_COEF_B*(float)rgb[offset+2];
    		offset += 3;
    	}
    } else {
    	for (idx = 0; idx < idxMax; idx++) {
    		rgb[idx] = RGB2GRAY_COEF_R*(float)rgb[offset] +
    					RGB2GRAY_COEF_G*(float)rgb[offset+1] +
                    	RGB2GRAY_COEF_B*(float)rgb[offset+2];
    		offset += 3;
    	}
    }
}
