#!/usr/bin/python3

import sys
import math

import json
from lxml import etree

usage = """
Adds delays from json file to flat pisdf.
./addDelays <flat.pi> <res.json>
"""

class EdgeIR:
    def __init__(self, prodActName, prodPortName, consActName, consPortName, delayIBM):
        self.prodActName = prodActName
        self.prodPortName = prodPortName

        self.consActName = consActName
        self.consPortName = consPortName

        self.delayIBM = delayIBM


## cannot work now since res.json lacks port names
def parseResJson(fileJson):
    listEdgesIR = []
    dictRes = None
    with open(fileJson, 'r') as f:
        dictRes = json.load(f)

    buffers = dictRes['bufferEntities']

    for buf in buffers:
        edge = EdgeIR(buf['producer'], buf['portIn'], buf['consumer'], buf['portOut'], float(buf['initValue']))
        listEdgesIR.append(edge)
        
    return listEdgesIR
    

def lcm(a, b):
    if a > b:
        return (a//math.gcd(a,b))*b
    return (b//math.gcd(a,b))*a

def simpleXPathWithName(path, prefix):
    subparts = path.split("/")
    auxPrefix = "/"+prefix
    return auxPrefix.join(subparts)

## closest value, being a multiple of cons and prod
## or multiple of repetition vector*prod or cons ?
def convertDelay(delayIBM, cons, prod):
    lcmpc = lcm(cons, prod)
    nbLcm = delayIBM / lcmpc
    return int(round(nbLcm)) * lcmpc

def modifyPi(filePi, listEdgesIR):

    actNames = set()
    ## get actors of interest
    for edge in listEdgesIR:
        actNames.add(edge.prodActName)
        actNames.add(edge.consActName)

    dicoActDicoPortRate = {}
    for name in actNames:
        dicoActDicoPortRate[name] = {}
        
    ## parse actors to get prod and cons rates
    parser = etree.XMLParser(remove_blank_text=True)
    treePi = etree.parse(filePi, parser)
    rootPi = treePi.getroot()

    rootAppNSmap = rootPi.nsmap
    # print(rootAppNSmap)

    # from here: https://stackoverflow.com/questions/6920073/how-to-use-xpath-from-lxml-on-null-namespaced-nodes
    prefix = 'gmd:' #'{' + rootAppNSmap[None] + '}'
    rootAppNSmap['gmd'] = rootAppNSmap[None]
    rootAppNSmap.pop(None)

    for actor in treePi.xpath(simpleXPathWithName("/graphml/graph/node", prefix), namespaces=rootAppNSmap):
        shortName = actor.get("id") # already is shortName if flat
        kind = actor.get("kind")
        if kind == "actor" or  kind == "broadcast" or kind == "roundbuffer" or kind == "fork" or kind == "join":
            for port in actor.getchildren():
                port_kind = port.get("kind")
                if kind == "input" or kind == "output":
                    port_name = port.get("name")
                    port_expr = int(port.get("expr"))
                    dicoActDicoPortRate[shortName][port_name] = port_expr
        else:
            continue

        
    
    ## modify edges
    for edge in listEdgesIR:

        ## "SIFT_" TO REMOVE
        
        # select edge
        rawQuery = "/graphml/graph/edge[@source='" + edge.prodActName +  "' and @target='" + edge.consActName + \
                                                  "' and @sourceport='" + edge.prodPortName + "' and @targetport='" + edge.consPortName + \
                                                  "']"

        edges = treePi.xpath(simpleXPathWithName(rawQuery, prefix), namespaces=rootAppNSmap)
        if len(edges) != 1:
            raise Exception("None, or more than one corresponding edge. Query: {}".format(rawQuery))
        stop = False
        for data in edges[0].getchildren():
            key = data.get("key")
            if key == "delay":
                stop = True
        if stop:
            ## we ignore places where delays are already set
            continue
        
        ## add delay child and create delay actor
        name = edge.prodActName + "_" + edge.prodPortName + "__" + edge.consActName + "_" + edge.consPortName
        print("SET {}".format(name))


        rawQuery = "/graphml/graph/node[@id='" + edge.prodActName +  "']/port[@name='" + edge.prodPortName + "']"
        prods = treePi.xpath(simpleXPathWithName(rawQuery, prefix), namespaces=rootAppNSmap)
        if len(prods) != 1:
            raise Exception("None, or more than one corresponding producer. Query: {}".format(rawQuery))
        prod = prods[0].attrib["expr"]
        
        rawQuery = "/graphml/graph/node[@id='" + edge.consActName +  "']/port[@name='" + edge.consPortName + "']"
        conss = treePi.xpath(simpleXPathWithName(rawQuery, prefix), namespaces=rootAppNSmap)
        if len(conss) != 1:
            raise Exception("None, or more than one corresponding consumer. Query: {}".format(rawQuery))
        cons = conss[0].attrib["expr"]
        
        expr = convertDelay(float(edge.delayIBM), int(prod), int(cons))
        
        edges[0].attrib["expr"] = str(expr)
        newData = etree.SubElement(edges[0],"data")
        newData.attrib["key"] = "delay"
        newData.text = name
        
        rawQuery = "/graphml/graph"
        graph = treePi.xpath(simpleXPathWithName(rawQuery, prefix), namespaces=rootAppNSmap)
        rawQuery = "/graphml/graph/node"
        nodes = treePi.xpath(simpleXPathWithName(rawQuery, prefix), namespaces=rootAppNSmap)
        newNode = etree.Element("node")
        graph[0].insert(1+len(nodes), newNode)
        
        newNode.attrib["getter"] = ""
        newNode.attrib["setter"] = ""
        newNode.attrib["expr"] = str(expr)
        newNode.attrib["kind"] = "delay"
        newNode.attrib["level"] = "local"
        newNode.attrib["id"] = name

        newPortIn = etree.SubElement(newNode, "port")
        newPortIn.attrib["annotation"] = "NONE"
        newPortIn.attrib["expr"] = str(prod)
        newPortIn.attrib["kind"] = "input"
        newPortIn.attrib["name"] = edge.consPortName
        
        newPortOut = etree.SubElement(newNode, "port")
        newPortOut.attrib["annotation"] = "NONE"
        newPortOut.attrib["expr"] = str(cons)
        newPortOut.attrib["kind"] = "output"
        newPortOut.attrib["name"] = edge.prodPortName

        
    with open("edited_"+filePi, 'wb') as newFile:
        newFile.write(etree.tostring(rootPi, pretty_print=True, encoding="UTF-8", xml_declaration=True))

    return

        
if __name__ == '__main__':
    if (len(sys.argv) < 3):
        print(usage)
        exit(1)
    filePi = sys.argv[1]
    fileJson = sys.argv[2]
    
    if (not filePi.endswith(".pi")):
        raise Exception("Graph file to parse must be in .pi format.")
    if (not fileJson.endswith(".json")):
        raise Exception("Result file to parse must be in .json format.")

    ## returns list of EdgeIR
    listEdgesIR = parseResJson(fileJson)

    modifyPi(filePi, listEdgesIR)

    
