#!/bin/bash


preesmDir="/home/ahonorat/Documents/logiciels/sources/preesm_3.18.0"

preesmProjectDir="/home/ahonorat/Documents/dev/2019_dac/data_preesm"


declare -A mapProbToGraph=(
    ["prediction"]="prediction_flat.pi"
    ["prediction_odroid"]="prediction_flat.pi"
    ["sift_4k"]="extract4K_flat.pi"
    ["sift_medium"]="extract_pmedium_flat.pi"
    ["sift_medium_odroid"]="extract_pmedium_flat.pi"
    ["sift_normal"]="extract_flat.pi"
    ["sift_normal_odroid"]="extract_flat.pi"
    ["stereo"]="stereo_top_flat.pi"
    ["stereo_odroid"]="stereo_top_flat.pi"
    ["stereo_plarge"]="stereo_top_plarge_flat.pi"
    ["stereo_plarge_odroid"]="stereo_top_plarge_flat.pi"
    ["training"]="training_flat.pi"
    ["training_odroid"]="training_flat.pi"
)

declare -A mapProbToScenar=(
    ["prediction"]="reinforcement/4corePrediction_flattened.scenario"
    ["prediction_odroid"]="reinforcement/OdroidPrediction_flattened.scenario"
    ["sift_4k"]="SIFT/extract4KSIFT_4Corex86_flattened.scenario"
    ["sift_medium"]="SIFT/extractSIFT_4Corex86_pmedium_flattened.scenario"
    ["sift_medium_odroid"]="SIFT/extractSIFT_odroid_pmedium_flattened.scenario"
    ["sift_normal"]="SIFT/extractSIFT_4Corex86_flattened.scenario"
    ["sift_normal_odroid"]="SIFT/extractSIFT_odroid_flattened.scenario"
    ["stereo"]="stereoGG/stereoGG_4CoreX86_flattened.scenario"
    ["stereo_odroid"]="stereoGG/stereoGG_Odroid_flattened.scenario"
    ["stereo_plarge"]="stereoGG/stereoGG_4CoreX86_plarge_flattened.scenario"
    ["stereo_plarge_odroid"]="stereoGG/stereoGG_Odroid_plarge_flattened.scenario"
    ["training"]="reinforcement/4coreTraining_flattened.scenario"
    ["training_odroid"]="reinforcement/OdroidTraining_flattened.scenario"
)


## name of the problem (without _app.dat or .json suffix)
IBMjsonProblem=${1}
IBMjsonSchedule=${2}


if [[ -z ${IBMjsonSchedule} || -z ${IBMjsonProblem} ]]
then
    echo "/!\ Usage: ./runIBMjsonPreesm.bash <IBM json problem name> <IBM json schedule file>"
    echo "Available problem names:"
    for name in "${!mapProbToGraph[@]}"; do echo "- ${name}"; done
    exit 1
fi



## get flat pi

flatPi=${mapProbToGraph[${IBMjsonProblem}]}

if [[ -z ${flatPi} ]]
then
    echo "/!\ No corresponding problem file has been found."
    echo "Available problem names:"
    for name in "${!mapProbToGraph[@]}"; do echo "- ${name}"; done
    exit 1
fi

## add delays in schedule.json file

echo "Adding delays to file: ${preesmProjectDir}/generatedData/flatpis/${flatPi}"
cp ${IBMjsonSchedule} ${preesmProjectDir}/generatedData/jsons/schedule.json
cd ${preesmProjectDir}/generatedData/flatpis/
python3 addDelays.py ${flatPi} ../jsons/schedule.json

ret=$?
if [ $ret -ne 0 ]; then
     echo "Delay addition failed, abandon."
     exit 1
fi

echo "Finish delay addition, reentering repertory:"
cd -
mv ${preesmProjectDir}/generatedData/flatpis/${flatPi} ${preesmProjectDir}/generatedData/flatpis/ori_${flatPi}
mv ${preesmProjectDir}/generatedData/flatpis/edited_${flatPi} ${preesmProjectDir}/generatedData/flatpis/${flatPi}


## run preesm

scenar=${mapProbToScenar[${IBMjsonProblem}]}

echo "Calling Preesm:"
${preesmProjectDir}/commandLinePreesm.sh "${preesmDir}" "${preesmProjectDir}" "CodegenIBM.workflow" "${scenar}"


echo "Removing temp files."
mv ${preesmProjectDir}/generatedData/flatpis/ori_${flatPi} ${preesmProjectDir}/generatedData/flatpis/${flatPi}
rm ${preesmProjectDir}/generatedData/jsons/schedule.json
rm velocity.log
