#!/usr/bin/python3

import sys
import csv

## ref is my Intel® Core™ i7-7820HQ, with a frequency range of 2.90 - 3.90 GHz, we take 3.9

factorA7 = 3.9/1.4 # 1.4 GHz
factorA15 = 3.9/2.0 # 2.0 GHz

nameA7 = "ARM_CortexA7"
nameA15 = "ARM_CortexA15"

usage = """
First argument must be the csv file with x86 timings.
"""

def parseCsv(fileCsv):
    dicoIn = {}
    with open(fileCsv, newline='') as csvfile:
        creader = csv.reader(csvfile, delimiter=';', quotechar='"')
        for row in creader:
            if len(row) != 2:
                raise Exception("Timings are not correct (2 columns expected).\n"+";".join(row))
            dicoIn[row[0]] = row[1]

    if dicoIn.get("Actors", "") != "x86":
        raise Exception("Only timings from x86 architectures are supported.")
    del dicoIn["Actors"]
    
    with open(fileCsv[0:-4] + "_odroid.csv", 'w', newline='') as csvfile:
        cwriter = csv.writer(csvfile, delimiter=';', quotechar='"', quoting=csv.QUOTE_MINIMAL)
        cwriter.writerow(['Actors', nameA7, nameA15])
        for key, value in dicoIn.items():
            cwriter.writerow([key, round(int(value)*factorA7), round(int(value)*factorA15)])


if __name__ == '__main__':
    if (len(sys.argv) < 2):
        print(usage)
        exit(1)
    fileCsv = sys.argv[1]
    if (fileCsv.endswith("_odroid.csv")):
        exit(0)
    if (not fileCsv.endswith(".csv")):
        raise Exception("Timings file to parse must be in .csv format.")

    parseCsv(fileCsv)
