#!/usr/bin/python3


from ibmdatagen import *

# import ibmdatagen.common as im
# import ibmdatagen.parsers as im
# import ibmdatagen.exporters as im


import sys
import math

usage = """
First argument must be the configure file.
"""

scenario = ""
slam = ""
application = ""
brv = ""
outfolder = "../data_ibm"
timingFactor = 1.0
localFactors = {}
wCN = 1.0

def parseLocalTiming(localFactors, localTimingFile):
    with open(localTimingFile, 'r') as fTiming:
        for line in fTiming:
            ar = line.split(' ')
            if (len(ar) < 3):
                raise Exception("Incorrect local timing file (actor name/initial firings/targeted firings).")
            original = int(ar[1])
            targeted = int(ar[2])
            if (targeted % original != 0):
                raise Exception("Incorrect local timing file: initial firings must be a divisor of targeted firings.")
            localFactors[ar[0]] = (targeted//original, original)
            

def parseConfig(fileCi):
    global scenario
    global slam
    global application
    global brv
    global timingFactor
    global outfolder
    global wCN
    with open(fileCi, 'r') as fConfig:
        for line in fConfig:
            if not line.rstrip() or line.startswith('#'):
                continue
            ar = line.split(' ')
            if (len(ar) < 2):
                raise Exception("Incorrect configuration file.")
            ar[1] = ar[1].strip()
            if ar[0] == "scenar:":
                scenario = ar[1]
                continue
            if ar[0] == "slam:":
                slam = ar[1]
                continue
            if ar[0] == "app:":
                application = ar[1]
                continue
            if ar[0] == "outfolder:":
                outfolder = ar[1]
                continue
            if ar[0] == "brv:":
                brv = ar[1]
                continue
            if ar[0] == "weightedCoreNumber:":
                wCN = float(ar[1])
                continue
            if ar[0] == "globalTimingFactor:":
                try:
                    timingFactor = float(ar[1])
                except ValueError as e:
                    print(e)
                    print("timingFactor, if specified, must be a float.")
                    exit(1)
                continue
            if ar[0] == "actorTimingFactors:":
                localTimingFile = ar[1]
                if localTimingFile:
                    if (not localTimingFile.endswith(".ltf")):
                        raise Exception("Local timing factors file must be in ltf format.")
                    parseLocalTiming(localFactors, localTimingFile)
                continue

    if (not application.endswith(".graphml") and not application.endswith(".pi")):
        raise Exception("Application file to parse must be in PREESM graphml or pi format.")
    if (not scenario.endswith(".scenario")):
        raise Exception("Scenario file to parse must be in PREESM scenario format.")
    if (not slam.endswith(".slam")):
        raise Exception("Architecture file to parse must be in PREESM slam format.")

    if (not outfolder.endswith("/")):
        outfolder = outfolder+"/"
        

if __name__ == '__main__':
    if (len(sys.argv) < 2):
        print(usage)
        exit(1)
    fileCi = sys.argv[1]

    if (not fileCi.endswith(".ci")):
        raise Exception("Configuration file to parse must be in .ci format.")

    parseConfig(fileCi)

    # give other functions instead of float in order to divide sizes/times/speeds

    # For OPL
    # scenario: timings in ns but wanted in µs,
    # app: sizes in Bytes but wanted in KBytes
    # archi: size in KBytes but wanted in KBytes and speed in KBytes per second but wanted in KBytes per µs
    scenarTimeFloatConverterOPL = lambda x: math.ceil(x/1000)
    appSizeIntConverterOPL = lambda x: float(x/1000)
    archiSizeStrConverterOPL = lambda x: float(x)
    archiSpeedStrConverterOPL = lambda x: float(x)/1000000

    props = common.Props(True, scenarTimeFloatConverterOPL, appSizeIntConverterOPL, archiSizeStrConverterOPL, archiSpeedStrConverterOPL, timingFactor, localFactors, wCN)

    scenar = parsers.parseScenar(props, scenario)
    app = parsers.parseApp(props, application, brv, scenar)
    archi = parsers.parseArchi(props, slam)

    newAppDat = outfolder + fileCi.split("/")[-1].replace(".ci","_app.dat")
    newArchiDat = outfolder + fileCi.split("/")[-1].replace(".ci", "_archi.dat")
    exporters.exportDatApp(newAppDat, newArchiDat, scenar, app, archi)


    
    # For Java
    # scenario: timings in ns but wanted in µs,
    # app: sizes in Bytes and wanted in Bytes
    # archi: size in KBytes and wanted in Bytes and speed in KBytes per second but wanted in Bytes per µs
    scenarTimeFloatConverterAOW = scenarTimeFloatConverterOPL
    appSizeIntConverterAOW = lambda x: int(x)
    archiSizeStrConverterAOW = lambda x: int(x)*1000
    archiSpeedStrConverterAOW = lambda x: math.ceil(float(x)/1000)

    props = common.Props(True, scenarTimeFloatConverterAOW, appSizeIntConverterAOW, archiSizeStrConverterAOW, archiSpeedStrConverterAOW, timingFactor, localFactors, wCN)
    scenar = parsers.parseScenar(props, scenario)
    app = parsers.parseApp(props, application, brv, scenar)
    archi = parsers.parseArchi(props, slam)

    newAppJson = outfolder + fileCi.split("/")[-1].replace(".ci",".json")
    exporters.exportJsonApp(newAppJson, scenar, app, archi)
