from .common import *

import json
import re

def exportDatApp(fileAppName, fileArchiName, scenar, app, archi):
    dicoActorId = app.dicoActorId
    dicoPortsId = app.dicoPortsId
    counterPorts = app.counterPorts
    counterEdges = app.counterEdges
    counterActors = app.counterActors
    
    with open(fileAppName, 'w') as fAppDat:
        fAppDat.write("minII = {:.3f};\n".format(app.minII))
        
        fAppDat.write("rawActors = {\n")
        for idx, (key, value) in enumerate(dicoActorId.items()):
            fAppDat.write("\t<{}, \"{}\", \"{}\", {}, {}>".format(value['id'], value['shortName'], value['kind'], value['repeat'], value.get('timing', 0)))
            if (idx + 1) == counterActors:
                fAppDat.write("\n")
            else:
                fAppDat.write(",\n")
        fAppDat.write("};\n")
    
        fAppDat.write("\nrawPorts = {\n")
        cPorts = 0
        for key, value in dicoActorId.items():
            for keyPort, valuePort in value['ports'].items():
                cPorts += 1
                fAppDat.write("\t<{}, {}, \"{}\", {}>".format(value['id'], valuePort['id'], keyPort, 1)) # last argument could be valuePort['tokenSize']
                if cPorts  == counterPorts:
                    fAppDat.write("\n")
                else:
                    fAppDat.write(",\n")
        fAppDat.write("};\n")
    
        fAppDat.write("\nrawPrecedences = {\n")
        for idx, dic in enumerate(dicoPortsId):
            fAppDat.write("\t<{}, {}, {}, {}, {}, {}, {}, {}, {}, {}>".
                          format(dic['fromAgent'], dic['fromPort'], dic['toAgent'], dic['toPort'], dic['prod'], dic['cons'], dic['delay'], dic['lcm'], dic['min'], dic['max']))
            if (idx + 1) == counterEdges:
                fAppDat.write("\n")
            else:
                fAppDat.write(",\n")
        fAppDat.write("};\n")
        
        
    if app.srdagGraph != None:
        with open(fileAppName.replace('.dat', '_srdag.dat'), 'w') as fSrdagDat:
            pattern = re.compile(r'(\S+)_(\d+)')
            fSrdagDat.write("srdagPrecedences = {\n")
            cEdges = 0
            counterEdges = len(app.srdagGraph.edges)
            for (u,v) in app.srdagGraph.edges:
                cEdges += 1
                msrc = pattern.match(u)
                mtgt = pattern.match(v)
                fSrdagDat.write("\t<{}, {}, \"{}\", {}, {}, \"{}\">".format(
                    app.mapActorShortNameId[msrc.group(1)], int(msrc.group(2)), u, app.mapActorShortNameId[mtgt.group(1)], int(mtgt.group(2)), v))
                if cEdges == counterEdges:
                    fSrdagDat.write("\n")
                else:
                    fSrdagDat.write(",\n")
            fSrdagDat.write("};\n")
                
        
    dicoArchiId = archi.dicoArchiId
    dicoMem = archi.dicoMem
    dicoProc = archi.dicoProc
    links = archi.links    
        
    linksMem = set()
    linksProcMem = set()
    for x,y,z in links:
        if x in dicoProc:
            linksProcMem.add((x,y))
        elif y in dicoProc:
            linksProcMem.add((y,x))
        else:
            linksMem.add((x,y,z))

    with open(fileArchiName, 'w') as fArchiDat:
        fArchiDat.write("processors = {\n")
        for idx, (key, value) in enumerate(dicoProc.items()):
            fArchiDat.write("\t<{}, \"{}\", {}, {}>".format(dicoArchiId[key], key, value['size'], 1))
            if (idx + 1) == len(dicoProc):
                fArchiDat.write("\n")
            else:
                fArchiDat.write(",\n")
        fArchiDat.write("};\n")

        fArchiDat.write("\nmemories = {\n")
        for idx, (key, value) in enumerate(dicoMem.items()):
            fArchiDat.write("\t<{}, \"{}\", {}, {}>".format(dicoArchiId[key], key, value['size'], value['speed']))
            if (idx + 1) == len(dicoMem):
                fArchiDat.write("\n")
            else:
                fArchiDat.write(",\n")
        fArchiDat.write("};\n")

        fArchiDat.write("\nproc2mem = {\n")
        for idx, (proc, mem) in enumerate(linksProcMem):
            fArchiDat.write("\t<{}, {}>".format(dicoArchiId[proc], dicoArchiId[mem]))
            if (idx + 1) == len(linksProcMem):
                fArchiDat.write("\n")
            else:
                fArchiDat.write(",\n")
        fArchiDat.write("};\n")

        fArchiDat.write("\nmem2mem = {\n")
        for idx, (x, y, z) in enumerate(linksMem):
            fArchiDat.write("\t<{}, {}, {}>".format(dicoArchiId[x], dicoArchiId[y], z))
            if (idx + 1) == len(linksMem):
                fArchiDat.write("\n")
            else:
                fArchiDat.write(",\n")
        fArchiDat.write("};\n")

    

def exportJsonApp(fileName, scenar, app, archi):

    dicoActorId = app.dicoActorId
    dicoPortsNm = app.dicoPortsNm
    dicoTimings = scenar.dicoTimings

    dicoJson = {'application': {'kind': app.appKind, 'UName': app.appName, 'actors': [], 'precedences': []},
                'architecture': {'version': archi.slamVersion, 'UName': archi.archiName, 'processingUnits': [], 'memories': [], 'connections': []},
                'executions': [], 'minII': "{:.3f}".format(app.minII)}

    dicoActors = {}
    
    for value in dicoActorId.values():
        shortName = value['shortName']
        dicoActor = {'UName': shortName, 'kind': value['kind'], 'repeat': value['repeat']}
        dicoActors[shortName] = dicoActor
        dicoJson['application']['actors'].append(dicoActor)
    for dic in dicoPortsNm:
        dicoJson['application']['precedences'].append({'consumer': dicoActors[dic['toAgent']], 'producer': dicoActors[dic['fromAgent']],
                                                       'consumerPort': dic['toPort'], 'producerPort': dic['fromPort'],
                                                       'consumptionSize': dic['cons'], 'productionSize': dic['prod'],
                                                       'delay': dic['delay'], 'lcm': dic['lcm'], 'min': dic['min'], 'max': dic['max']})
    for dic in dicoTimings:
        dic['actor'] = dicoActors[dic['actor']]
        dicoJson['executions'].append(dic)

    for key, value in archi.dicoProc.items():
        dicoJson['architecture']['processingUnits'].append({'UName': key, 'cores': value['size']})
    for key, value in archi.dicoMem.items():
        dicoJson['architecture']['memories'].append({'UName': key, 'maxCapacity': value['size'], 'speed': value['speed'], 'type': value['spiritName']})
    dicoMem = archi.dicoMem
    dicoProc = archi.dicoProc
    for (x,y,z) in archi.links:
        dicoCon = {'memories': [], 'processingUnits': []}
        if x in dicoMem:
            value = dicoMem[x]
            dicoCon['memories'].append({'UName': x, 'maxCapacity': value['size'], 'speed': value['speed'], 'type': value['spiritName']})
        else:
            value = dicoProc[x]
            dicoCon['processingUnits'].append({'UName': x, 'cores': value['size']})
        if y in dicoMem:
            value = dicoMem[y]
            dicoCon['memories'].append({'UName': y, 'maxCapacity': value['size'], 'speed': value['speed'], 'type': value['spiritName']})
        else:
            value = dicoProc[y]
            dicoCon['processingUnits'].append({'UName': y, 'cores': value['size']})
        dicoJson['architecture']['connections'].append(dicoCon)

    if app.srdagGraph != None:
        listPred = []
        pattern = re.compile(r'(\S+)_(\d+)')
        for (u,v) in app.srdagGraph.edges:
            dicoPred = {}
            msrc = pattern.match(u)
            dicoPred['srcActor'] = msrc.group(1)
            dicoPred['srcFiringName'] = u
            dicoPred['srcFiringInstance'] = msrc.group(2)
            mtgt = pattern.match(v)
            dicoPred['tgtActor'] = mtgt.group(1)
            dicoPred['tgtFiringName'] = v
            dicoPred['tgtFiringInstance'] = mtgt.group(2)
            listPred.append(dicoPred)
        dicoJson['srdagPrecedences'] = listPred

        
    with open(fileName, 'w') as fAppJson:
        json.dump(dicoJson, fAppJson, indent = 4)

