from .common import *

from lxml import etree, objectify
from sortedcontainers import SortedList

import itertools
import math
import networkx as nx

import os.path


def lcm(a, b):
    if a > b:
        return (a/math.gcd(a,b))*b
    return (b/math.gcd(a,b))*a

def parseArchi(props, archiFile):
## will be easier to remove this code and storing the result for the most common archi (x86 and Odroid), than to maintain it
    
    treeSlam = etree.parse(archiFile)       
    rootSlam = treeSlam.getroot()    
    rootSlamNSmap = rootSlam.nsmap
    #print(rootSlamNSmap)
    #print(rootSlam.prefix)
    spiritNS = "spirit:"
    slamNS = "slam:"

    maxSlamValue = 2**31
    
    for hier in treeSlam.xpath(simpleXPathWithName("/design/hierConnections", spiritNS), namespaces=rootSlamNSmap):
        if hier.getchildren():
            print("Error! --> hierarchical connections in slam are not allowed in this script.")
            exit(1)

    archiName = rootSlam.find(simpleXPathWithName("./name", spiritNS), namespaces=rootSlamNSmap).text
    slamVersion = rootSlam.find(simpleXPathWithName("./version", spiritNS), namespaces=rootSlamNSmap).text
            
    dicoCP = {}
    dicoCPref = {} # {'cpInstanceName': 'cpSpiritName'}
    listUndirLN = []

    sizeStrConverter = props.archiSizeStrConverter
    speedStrConverter = props.archiSpeedStrConverter
    clusterProc = props.clusterProc
    
    for vendorExt in treeSlam.xpath(simpleXPathWithName("/design/vendorExtensions", spiritNS), namespaces=rootSlamNSmap):
        for child in vendorExt.getchildren():
            if child.tag.endswith("componentDescriptions"):
                cpNSmap = child.nsmap
                for cpDesc in child.xpath("./" + slamNS + "componentDescription", namespaces=cpNSmap):
                    cpRefName = cpDesc.get("{"+cpNSmap['slam']+"}componentRef") # corresponds to type in json for AOW
                    dicoCP[cpRefName] = {}
                    cpType = cpDesc.get("{"+cpNSmap['slam']+"}componentType")
                    dicoCP[cpRefName]['cpType'] = cpType
                    if cpType.endswith('ComNode'): # modify speed with units
                        speedStr = cpDesc.get("{"+cpNSmap['slam']+"}speed")
                        if speedStr == "":
                            speed = maxSlamValue # max currently supported in slam
                        else:
                            speed = speedStrConverter(speedStr)
                        dicoCP[cpRefName]['speed'] = speed
                    elif cpType == "Mem":
                        size = sizeStrConverter(cpDesc.get("{"+cpNSmap['slam']+"}size"))
                        dicoCP[cpRefName]['size'] = size
                    dicoCP[cpRefName]['nbInstances'] = 0
            elif child.tag.endswith("linkDescriptions"):
                lnNSmap = child.nsmap
                for lnDesc in child.xpath("./" + slamNS + "linkDescription", namespaces=lnNSmap):
                    lnType = lnDesc.get("{"+lnNSmap['slam']+"}linkType")
                    lnDir = lnDesc.get("{"+lnNSmap['slam']+"}directedLink")
                    if lnType == "DataLink" and lnDir == "undirected":
                        listUndirLN.append(lnDesc.get("{"+lnNSmap['slam']+"}referenceId"))
    # print(str(dicoCP))
    # print(listUndirLN)
            
    for component in treeSlam.xpath(simpleXPathWithName("/design/componentInstances/componentInstance", spiritNS), namespaces=rootSlamNSmap):
        nameEl = component.find(simpleXPathWithName("./instanceName", spiritNS), namespaces=rootSlamNSmap)
        name = nameEl.text 
        refEl = component.find(simpleXPathWithName("./componentRef", spiritNS), namespaces=rootSlamNSmap)
        ref = refEl.get("{"+rootSlamNSmap['spirit']+"}name")
        dicoCP[ref]['nbInstances'] = dicoCP[ref]['nbInstances'] + 1
        dicoCPref[name] = ref
    # print(dicoCPref)

    dicoArchiId = {}
    dicoMem = {} # {'memInstanceName': {'speed': , 'size':, 'id': , 'spiritName': } }
    dicoProc = {} # {'procInstanceName': {'speed': , 'size':, 'id': , 'spiritName': } }
    dicoComNode = {} # {'comNodeInstanceName': {'speed': , 'size':, 'spiritName': } }
    
    dicoClusters = {}
    for cpInsName, cpRefName in dicoCPref.items():
        if dicoCP[cpRefName]['cpType'] == "Mem":
            dicM = {'spiritName': cpRefName, 'id': len(dicoMem)+1, 'size': dicoCP[cpRefName]['size'], 'speed': maxSlamValue}
            dicoArchiId[cpInsName] = dicM['id']
            dicoMem[cpInsName] = dicM
        elif dicoCP[cpRefName]['cpType'] == "Operator":
            dicP = {'spiritName': cpRefName, 'id': len(dicoProc)+1, 'size': 1, 'speed': 1}
            dicoArchiId[cpInsName] = dicP['id']
            if clusterProc:
                dicP['size'] = dicoCP[cpRefName]['nbInstances']
                clusterName = cpRefName+'_cluster'
                dicoArchiId[clusterName] = dicoArchiId.get(clusterName, len(dicoClusters) + 1)
                dicoProc[clusterName] = dicP
                dicoClusters[clusterName] = cpRefName
            else:
                dicoProc[cpInsName] = dicP
        elif dicoCP[cpRefName]['cpType'].endswith('ComNode'):
            dicCN = {'spiritName': cpRefName, 'speed': dicoCP[cpRefName]['speed']}
            dicCN['size'] = dicoCP[cpRefName]['nbInstances']
            dicoComNode[cpInsName] = dicCN
    for key, value in dicoClusters.items():
        dicoCPref[key] = value
    # print(dicoMem)
    # print(dicoProc)
    # print(dicoComNode)
            
    # links ...
    links = set() # tuple (instanceName, instanceName)
    for component in treeSlam.xpath(simpleXPathWithName("/design/interconnections/interconnection", spiritNS), namespaces=rootSlamNSmap):
        nameEl = component.find(simpleXPathWithName("./name", spiritNS), namespaces=rootSlamNSmap)
        name = nameEl.text
        if name in listUndirLN:
            els = component.findall(simpleXPathWithName("./activeInterface", spiritNS), namespaces=rootSlamNSmap)
            el0 = els[0].get("{"+rootSlamNSmap['spirit']+"}componentRef")
            el1 = els[1].get("{"+rootSlamNSmap['spirit']+"}componentRef")
            if el0 != el1:
                el0spir = dicoCPref[el0]
                el1spir = dicoCPref[el1]
                if clusterProc:
                    el0cluster = el0spir+'_cluster'
                    if el0cluster in dicoProc:
                        el0 = el0cluster
                    el1cluster = el1spir+'_cluster'
                    if el1cluster in dicoProc:
                        el1 = el1cluster
                speed = maxSlamValue
                if dicoCP[el0spir]['cpType'].endswith('ComNode'):
                    speed = min(speed, dicoComNode[el0]['speed'])
                    if dicoCP[el1spir]['cpType'] == "Mem":
                        dicoMem[el1]['speed'] == min(speed, dicoMem[el1]['speed'])
                if dicoCP[el1spir]['cpType'].endswith('ComNode'):
                    speed = min(speed, dicoComNode[el1]['speed'])
                    if dicoCP[el0spir]['cpType'] == "Mem":
                        dicoMem[el0]['speed'] == min(speed, dicoMem[el0]['speed'])
                if el0 < el1:
                    links.add((el0, el1, speed))
                else:
                    links.add((el1, el0, speed))
    #print(links)

    for cpInsName, value in dicoComNode.items():
        lLinksToReplace = [(x,y,z) for (x,y,z) in links if x == cpInsName or y == cpInsName]
        dests = set()
        for (x,y,z) in lLinksToReplace:
            if x == cpInsName:
                dests.add(y)
            elif y == cpInsName:
                dests.add(x)
            links.remove((x,y,z))

        oneIsRAM = None
        for x in dests:
            xspir = dicoCPref[x]
            if (x == "RAM" or x == "shared_mem") and dicoCP[xspir]['cpType'] == "Mem":
                oneIsRAM = x

        if oneIsRAM and dicoCP[dicoCPref[cpInsName]]['cpType'] == "contentionComNode":
            speed = min(z, dicoMem[oneIsRAM]['speed'])
            dests.remove(oneIsRAM)
            for y in dests:
                yspir = dicoCPref[y]
                if dicoCP[yspir]['cpType'] == "Mem":
                    speed = min(speed, dicoMem[y]['speed'])
                elif dicoCP[yspir]['cpType'].endswith('ComNode'):
                    speed = min(speed, dicoComNode[y]['speed'])
                if y < oneIsRAM:
                    links.add((y,oneIsRAM,speed))
                else:
                    links.add((oneIsRAM,y,speed))
        
        else:
            for (x,y) in itertools.product(dests, repeat=2):
                xspir = dicoCPref[x]
                yspir = dicoCPref[y]
                speed = z # z is not modified since we process one comNode
                if x < y and (dicoCP[xspir]['cpType'] != 'processingUnit' or
                              dicoCP[yspir]['cpType'] != 'processingUnit'):
                    if dicoCP[xspir]['cpType'] == "Mem":
                        speed = min(speed, dicoMem[x]['speed'])
                    elif dicoCP[xspir]['cpType'].endswith('ComNode'):
                        speed = min(speed, dicoComNode[x]['speed'])
                    if dicoCP[yspir]['cpType'] == "Mem":
                        speed = min(speed, dicoMem[y]['speed'])
                    elif dicoCP[yspir]['cpType'].endswith('ComNode'):
                        speed = min(speed, dicoComNode[y]['speed'])
                    links.add((x,y,speed))
    #print(links)
    print("{} direct connections in archi".format(len(links)))
    return Archi(dicoArchiId, dicoMem, dicoProc, links, archiName, slamVersion)


class AppIR:
    def __init__(self, graph, dicoActorId, dicoPortsId, dicoPortsNm, mapActorIdShortName, mapActorShortNameId):
        self.counterActors = 0
        self.counterEdges = 0
        self.counterPorts = 0
        self.appKind = "sdf"
        self.appName = "sample"
        self.graph = graph

        self.srdagGraph = None

        self.dicoActorId = dicoActorId
        self.dicoPortsId = dicoPortsId
        self.dicoPortsNm = dicoPortsNm
        self.mapActorIdShortName = mapActorIdShortName
        self.mapActorShortNameId = mapActorShortNameId

        
def parseApp(props, appFile, brvFile, scenar):
    graph = nx.DiGraph()
    
    dicoActorId = {} # actually a dico of dicos: {'SIFT_actor1': {'id': 1, repeat': 4, 'kind': "executable", 'timing': 87687, 'shortName': "actor1", 'ports': {'in': {'tokenSize': 1, 'id': 1}}}
    dicoPortsId = [] # [{'fromAgent': 1, 'fromPort': 2, 'toAgent': 3, 'toPort': 4, 'prod': 987, 'cons': 876, 'delay': 567}]
    dicoPortsNm = [] # idem dicoPortsId with names instead of Id

    mapActorIdShortName = {}
    mapActorShortNameId = {}

    appRI = AppIR(graph, dicoActorId, dicoPortsId, dicoPortsNm, mapActorIdShortName, mapActorShortNameId)

    if appFile.endswith(".graphml"):
        parseGraphml(props, scenar, appFile, appRI)
    elif appFile.endswith(".pi") and brvFile:
        parsePi(props, scenar, appFile, brvFile, appRI)
    else:
        raise Exception("Incorrect ci file: .graphml, or .pi and brv.xml files are not provided.")

    nbEdgesN = len(appRI.graph.edges)
    print("Computing cycles ... ({} edges in simple graph)".format(nbEdgesN))
    
    listCycles = list(nx.simple_cycles(appRI.graph))
    listGcd = []
    setCyclesEdges = set()
    print("{} cycles in basis".format(len(listCycles)))
    for cycle in listCycles:
        strCycle = ""
        prevNode = cycle[-1]
        gcdCycleRepeat = dicoActorId[mapActorIdShortName[cycle[-1]]]['repeat']
        for node in cycle:
            #strCycle = strCycle + mapActorIdShortName[node] + " --> "
            repeat = dicoActorId[mapActorIdShortName[node]]['repeat']
            gcdCycleRepeat = math.gcd(gcdCycleRepeat, repeat)
            setCyclesEdges.add((prevNode, node))
            prevNode = node
        #print(strCycle)
        listGcd.append(gcdCycleRepeat)
        for node in cycle:
            shortName = mapActorIdShortName[node]
            repeat = dicoActorId[shortName]['repeat'] // gcdCycleRepeat
            minCycleRepeat = min(repeat, dicoActorId[shortName].get('minCycleRepeat', repeat))
            dicoActorId[shortName]['minCycleRepeat'] = minCycleRepeat
            
    maxCycleLengthMin = 0
    for (cycle, gcdCycleRepeat) in zip(listCycles, listGcd):
        cycleLengthTot = 0
        for node in cycle:
            shortName = mapActorIdShortName[node]
            repeat = dicoActorId[shortName]['minCycleRepeat']
            timing = dicoActorId[shortName]['timing']
            factor = max(repeat / props.weightedCoreNumber, 1)
            cycleLengthTot += timing*factor
        maxCycleLengthMin = max(maxCycleLengthMin, cycleLengthTot*gcdCycleRepeat)

    
    computationalTotTime = 0
    maxC = 0
    for value in appRI.dicoActorId.values():
        timing = value['timing']
        maxC = max(maxC, timing)
        computationalTotTime = computationalTotTime + value['repeat']*timing

    print("Tot max computational time: {}".format(computationalTotTime))
    minII = max(maxC, computationalTotTime/props.weightedCoreNumber, maxCycleLengthMin)
    print("Minimum II length: {:.3f}".format(minII))
    if computationalTotTime < maxCycleLengthMin:
        raise Exception("Wrong minII")
        
    for dico in appRI.dicoPortsId:
        src = dico['fromAgent']
        dst = dico['toAgent']
        if (src, dst) in setCyclesEdges:
            dico['max'] = dico['min']
    counterFixedSize = 0
    for dico in appRI.dicoPortsNm:
        src = appRI.mapActorShortNameId.get(dico['fromAgent'])
        dst = appRI.mapActorShortNameId.get(dico['toAgent'])
        if (src, dst) in setCyclesEdges:
            dico['max'] = dico['min']
            counterFixedSize = counterFixedSize + 1
    print("{} edges transformed, {} of fixed size.".format(appRI.counterEdges, counterFixedSize))

    return App(appRI.dicoActorId, appRI.dicoPortsNm, appRI.dicoPortsId, appRI.appName, appRI.appKind,
               appRI.counterActors, appRI.counterEdges, appRI.counterPorts,
               appRI.srdagGraph, appRI.mapActorShortNameId, minII)


def parseBrv(brvFile):
    treeBrv = etree.parse(brvFile)
    dicoBrv = {}
    for entry in treeBrv.xpath("/brv/nbRepeat"):
        name = entry.get("shortName")
        nbRepeat = int(entry.get("fullRV"))
        dicoBrv[name] = nbRepeat
    return dicoBrv
        
def parsePi(props, scenar, appFile, brvFile, appRI):

    dicoBrv = parseBrv(brvFile)
    dicoDelay = {}
    
    localFactors = props.actorTimingFactors
    sizeIntConverter = props.appSizeIntConverter

    dicoTimingsMax = scenar.dicoTimingsMax
    dicoSizes = scenar.dicoSizes
    
    treePi = etree.parse(appFile)
    rootPi = treePi.getroot()

    rootAppNSmap = rootPi.nsmap
    # print(rootAppNSmap)

    # from here: https://stackoverflow.com/questions/6920073/how-to-use-xpath-from-lxml-on-null-namespaced-nodes
    prefix = 'gmd:' #'{' + rootAppNSmap[None] + '}'
    rootAppNSmap['gmd'] = rootAppNSmap[None]
    rootAppNSmap.pop(None)

    for data in treePi.xpath(simpleXPathWithName("/graphml/graph/data", prefix), namespaces=rootAppNSmap):
        key = data.get('key')
        if key == 'name':
            appRI.appName = data.text

    sumRV = 0
    for actor in treePi.xpath(simpleXPathWithName("/graphml/graph/node", prefix), namespaces=rootAppNSmap):
        shortName = actor.get("id") # already is shortName if flat
        kind = actor.get("kind")
        if kind == "actor":
            kind = "executable"
        elif kind == "broadcast" or kind == "roundbuffer" or kind == "fork" or kind == "join":
            pass
        else:
            if kind == "delay":
                for port in actor.getchildren():
                    port_kind = port.get("kind")
                    if (port_kind == "input"):
                        dicoDelay[shortName] = int(port.get("expr"))
                        break
            continue
        appRI.counterActors += 1
        nbRepeat = dicoBrv.get(shortName, 0)
        oriNbRepeat = (localFactors.get(shortName, (1, nbRepeat)))[1]
        if (oriNbRepeat != nbRepeat):
            raise Exception("Incorrect local timing file: initial firings are wrong ({}: {}<>{})".format(shortName, oriNbRepeat, nbRepeat))
        nbRepeat = nbRepeat*(localFactors.get(shortName, (1, nbRepeat)))[0]
        sumRV = sumRV + nbRepeat
        time = dicoTimingsMax.get(shortName, 0)
        
        dicoPortsLocal = {}
        for port in actor.getchildren():
            port_kind = port.get("kind")
            if (port_kind == "input" or port_kind == "output"):
                appRI.counterPorts += 1
                port_name = port.get("name")
                dicoPortsLocal[port_name] = {'id': len(dicoPortsLocal)+1, 'expr': int(port.get("expr"))}

        appRI.dicoActorId[shortName] = {'id': appRI.counterActors, 'repeat': nbRepeat, 'kind': kind, 'timing': time, 'ports': dicoPortsLocal, 'shortName': shortName}
        appRI.mapActorIdShortName[appRI.counterActors] = shortName
        appRI.mapActorShortNameId[shortName] = appRI.counterActors
        appRI.graph.add_node(appRI.counterActors)
        
    print("{} actors transformed".format(appRI.counterActors))
    print("Tot nbFirings: {}".format(sumRV))
    
    for edge in treePi.xpath(simpleXPathWithName("/graphml/graph/edge", prefix), namespaces=rootAppNSmap):
        kind = edge.get("kind")
        if kind != "fifo":
            continue
        appRI.counterEdges += 1
        delay = 0
        delay_name = ""
        data_type = edge.get("type")
        fromAgent = edge.get("source")
        toAgent = edge.get("target")
        fromPort = edge.get("sourceport")
        toPort = edge.get("targetport")
        for data in edge.getchildren():
            if (data.get("key") == "delay"):
                delay_name = data.text
                delay = sizeIntConverter(dicoDelay[delay_name]*dicoSizes[data_type])
        appRI.dicoActorId[fromAgent]['ports'][fromPort]['tokenSize'] = dicoSizes[data_type]
        appRI.dicoActorId[toAgent]['ports'][toPort]['tokenSize'] = dicoSizes[data_type]
        rawProd = appRI.dicoActorId[fromAgent]['ports'][fromPort]['expr']*dicoSizes[data_type]
        rawCons = appRI.dicoActorId[toAgent]['ports'][toPort]['expr']*dicoSizes[data_type]
        prod = sizeIntConverter(rawProd)
        cons = sizeIntConverter(rawCons)
        lcmEdge = sizeIntConverter(lcm(rawProd, rawCons))
        maxDelay = sizeIntConverter(rawCons*appRI.dicoActorId[toAgent]['repeat'])        
        appRI.dicoPortsId.append({'fromAgent': appRI.dicoActorId[fromAgent]['id'], 'toAgent': appRI.dicoActorId[toAgent]['id'],
                                  'fromPort': appRI.dicoActorId[fromAgent]['ports'][fromPort]['id'],
                                  'toPort': appRI.dicoActorId[toAgent]['ports'][toPort]['id'],
                                  'prod': prod, 'cons': cons, 'delay': delay,
                                  'lcm': lcmEdge, 'min': delay, 'max': maxDelay})
        appRI.dicoPortsNm.append({'fromAgent': appRI.dicoActorId[fromAgent]['shortName'], 'toAgent': appRI.dicoActorId[toAgent]['shortName'],
                                  'fromPort': fromPort, 'toPort': toPort,
                                  'prod': prod, 'cons': cons, 'delay': delay,
                                  'lcm': lcmEdge, 'min': delay, 'max': maxDelay})
        appRI.graph.add_edge(appRI.dicoActorId[fromAgent]['id'], appRI.dicoActorId[toAgent]['id'])

    parsePiSRDAG(appFile, appRI)



def parsePiSRDAG(appFile, appRI):
    srdagFile = appFile.replace('flatpis', 'flatsrdags')

    if not os.path.isfile(srdagFile):
        print("No SRDAG file found.")
        return
    
    treePi = etree.parse(srdagFile)
    rootPi = treePi.getroot()

    rootAppNSmap = rootPi.nsmap
    # print(rootAppNSmap)

    # from here: https://stackoverflow.com/questions/6920073/how-to-use-xpath-from-lxml-on-null-namespaced-nodes
    prefix = 'gmd:' #'{' + rootAppNSmap[None] + '}'
    rootAppNSmap['gmd'] = rootAppNSmap[None]
    rootAppNSmap.pop(None)
    
    graph = nx.DiGraph()
    dicoActorKind = {}
    for actor in treePi.xpath(simpleXPathWithName("/graphml/graph/node", prefix), namespaces=rootAppNSmap):
        shortName = actor.get("id") # already is shortName if flat
        kind = actor.get("kind")
        graph.add_node(shortName)
        dicoActorKind[shortName] = kind
    
    for edge in treePi.xpath(simpleXPathWithName("/graphml/graph/edge[@kind='fifo']", prefix), namespaces=rootAppNSmap):
        source = edge.get('source')
        target = edge.get('target')
        graph.add_edge(source, target)

    for (sname, kind) in dicoActorKind.items():
        if kind == "init" or kind == "end" or (kind == "join" and sname.startswith('implode_')) or (kind == "fork" and sname.startswith('explode_')):
            if graph.nodes.get(sname, default=None) != None:
                for suc in graph.succ[sname].keys():
                    for pred in graph.pred[sname].keys():
                        #print("Adding connection between {} and {}".format(pred,suc))
                        graph.add_edge(pred,suc)
                graph.remove_node(sname)
                #print("Removing: {}".format(sname))                

    print("SRDAG contains {} edges!".format(len(graph.edges)))
    appRI.srdagGraph = graph
    

def parseGraphml(props, scenar, appFile, appRI):
    localFactors = props.actorTimingFactors
    sizeIntConverter = props.appSizeIntConverter

    dicoTimingsMax = scenar.dicoTimingsMax
    dicoSizes = scenar.dicoSizes
    
    treeGraphml = etree.parse(appFile)
    rootGraphml = treeGraphml.getroot()

    rootAppNSmap = rootGraphml.nsmap
    # print(rootAppNSmap)

    # from here: https://stackoverflow.com/questions/6920073/how-to-use-xpath-from-lxml-on-null-namespaced-nodes
    prefix = 'gmd:' #'{' + rootAppNSmap[None] + '}'
    rootAppNSmap['gmd'] = rootAppNSmap[None]
    rootAppNSmap.pop(None)


    # from here: https://stackoverflow.com/questions/18159221/remove-namespace-and-prefix-from-xml-in-python-using-lxml
    # newGraphml = appFile.replace(".graphml","_transformed.graphml")

    # for elem in rootGraphml.getiterator():
    #     if not hasattr(elem.tag, 'find'): continue  # (1)
    #     i = elem.tag.find('}')
    #     if i >= 0:
    #         elem.tag = elem.tag[i+1:]
        
    # objectify.deannotate(rootGraphml, cleanup_namespaces=True)
    # treeGraphml.write(newGraphml, pretty_print=True, xml_declaration=True, encoding='UTF-8')
    
    # treeGraphml = etree.parse(newGraphml)

    for data in treeGraphml.xpath(simpleXPathWithName("/graphml/graph/data", prefix), namespaces=rootAppNSmap):
        key = data.get('key')
        if key == 'name':
            appRI.appName = data.text
        elif key == 'kind':
            appRI.appKind = data.text
    if appRI.appKind != "sdf":
        print("Error: graphml must represent an sdf application")
        exit(1)

    sumRV = 0
    for actor in treeGraphml.xpath(simpleXPathWithName("/graphml/graph/node", prefix), namespaces=rootAppNSmap):
        appRI.counterActors += 1
        name = ""
        nbRepeat = 0
        kind = ""
        for data in actor.getchildren():
            if (data.get("key") == "name"):
                name = data.text
            if (data.get("key") == "nbRepeat"):
                nbRepeat = int(data.text)
            if (data.get("key") == "kind"):
                kind = data.text
        if kind == "vertex":
            kind = "executable"
        shortName = scenar.getShortName(name)
        oriNbRepeat = (localFactors.get(shortName, (1, nbRepeat)))[1]
        if (oriNbRepeat != nbRepeat):
            raise Exception("Incorrect local timing file: initial firings are wrong ({}: {}<>{})".format(shortName, oriNbRepeat, nbRepeat))
        nbRepeat = nbRepeat*(localFactors.get(shortName, (1, nbRepeat)))[0]
        sumRV = sumRV + nbRepeat
        time = dicoTimingsMax.get(shortName, 0)
        appRI.dicoActorId[shortName] = {'id': appRI.counterActors, 'repeat': nbRepeat, 'kind': kind, 'timing': time, 'ports': {}, 'shortName': shortName}
        appRI.mapActorIdShortName[appRI.counterActors] = shortName
        appRI.mapActorShortNameId[shortName] = appRI.counterActors
        appRI.graph.add_node(appRI.counterActors)
        
    print("{} actors transformed".format(appRI.counterActors))
    print("Tot nbFirings: {}".format(sumRV))        
        
    for edge in treeGraphml.xpath(simpleXPathWithName("/graphml/graph/edge", prefix), namespaces=rootAppNSmap):
        appRI.counterEdges += 1
        data_type = "void"
        fromAgent = scenar.getShortName(edge.get("source"))
        toAgent = scenar.getShortName(edge.get("target"))
        fromPort = scenar.getShortName(edge.get("sourceport"))
        toPort = scenar.getShortName(edge.get("targetport"))
        rawProd = 0
        rawCons = 0
        prod = 0
        cons = 0
        delay = 0
        for data in edge.getchildren():
            if (data.get("key") == "data_type"):
                data_type = data.text
        for data in edge.getchildren():
            if (data.get("key").startswith("edge_")):
                rawSize = int(data.text)*dicoSizes[data_type]
                size = sizeIntConverter(rawSize)
            if (data.get("key") == "edge_cons"):
                rawCons = rawSize
                cons = size
            if (data.get("key") == "edge_prod"):
                rawProd = rawSize
                prod = size
            if (data.get("key") == "edge_delay"):
                delay = size
        lcmEdge = sizeIntConverter(lcm(rawProd, rawCons))
        maxDelay = sizeIntConverter(rawCons*appRI.dicoActorId[toAgent]['repeat'])
        if (appRI.dicoActorId[fromAgent]['ports'].get(fromPort) == None):
            appRI.dicoActorId[fromAgent]['ports'][fromPort] = {'id': len(appRI.dicoActorId[fromAgent]['ports'])+1, 'tokenSize': dicoSizes[data_type]}
            appRI.counterPorts += 1
        if (appRI.dicoActorId[toAgent]['ports'].get(toPort) == None):
            appRI.dicoActorId[toAgent]['ports'][toPort] = {'id': len(appRI.dicoActorId[toAgent]['ports'])+1, 'tokenSize': dicoSizes[data_type]}
            appRI.counterPorts += 1
        appRI.dicoPortsId.append({'fromAgent': appRI.dicoActorId[fromAgent]['id'], 'toAgent': appRI.dicoActorId[toAgent]['id'],
                            'fromPort': appRI.dicoActorId[fromAgent]['ports'][fromPort]['id'],
                            'toPort': appRI.dicoActorId[toAgent]['ports'][toPort]['id'],
                            'prod': prod, 'cons': cons, 'delay': delay,
                            'lcm': lcmEdge, 'min': delay, 'max': maxDelay})
        appRI.dicoPortsNm.append({'fromAgent': appRI.dicoActorId[fromAgent]['shortName'], 'toAgent': appRI.dicoActorId[toAgent]['shortName'],
                            'fromPort': fromPort, 'toPort': toPort,
                            'prod': prod, 'cons': cons, 'delay': delay,
                            'lcm': lcmEdge, 'min': delay, 'max': maxDelay})
        appRI.graph.add_edge(appRI.dicoActorId[fromAgent]['id'], appRI.dicoActorId[toAgent]['id'])


def parseScenar(props, scenarioFile):
    ## parse actor possible prefixes according to scenario constraints
    ## and retrieve maximum timing of each vertex. It is always associated with 4 cores. (why?)

    timeFloatConverter = props.scenarTimeFloatConverter
    timingFactor = props.globalTimingFactor
    localFactors = props.actorTimingFactors
    
    treeScenario = etree.parse(scenarioFile)
    listPrefix = set()
    dicoTimingsMax = {}
    dicoTimings = []
    dicoSizes = {}
    
    for dataType in treeScenario.xpath("/scenario/simuParams/dataTypes/dataType"):
        dicoSizes[dataType.get("name")] = int(dataType.get("size"))
    print("Data sizes: "+str(dicoSizes))
        
    # not necessary anymore: names have been unified in the scenario (always the vertex path)
    for task in treeScenario.xpath("/scenario/constraints/constraintGroup/task"):
        name = task.get("name")
        parents = name.split("/")
        del parents[-1]
        if len(parents) == 0:
            continue
        del parents[0]
        nbParents = len(parents)
        if nbParents > 0:
            listPrefix.add((nbParents, "_".join(parents)+"_"))
    sortedListPrefix = SortedList(listPrefix, key = lambda p: -p[0])
    strPrefixes = " - ".join([s for l,s in sortedListPrefix])
    print("Prefixes: "+strPrefixes)

    
    for timing in treeScenario.xpath("/scenario/timings/timing"):
        name = timing.get("vertexname")
        parents = name.split("/")
        del parents[0]
        if len(parents) > 0:
            name = parents[-1]
        opname = timing.get("opname")
        if props.clusterProc:
            opname = opname + "_cluster"
        ftime = (float(timing.get("time"))*timingFactor)/(localFactors.get(name, (1, 1)))[0]
        time = timeFloatConverter(ftime)
        dicoTimingsMax[name] = max(time, dicoTimingsMax.get(name, 0))
        dicoTimings.append({'actor': name, 'processingUnit': {'UName': opname, 'cores': 4}, 'time': time})

    return Scenar(dicoTimings, dicoTimingsMax, dicoSizes, sortedListPrefix)

